from datetime import datetime
from typing import Dict, List

from openapi_client import ProtoExperimentPlotMetric, ProtoExperimentPlotFile
from savvihub import ArgumentException
from savvihub.cli.commands.volume import volume_file_copy_local_to_remote
from savvihub.constants import SAVVIHUB_IMAGES, SAVVIHUB_PLOTS_FILETYPE_IMAGE


class History:
    def __init__(self, experiment):
        self.experiment = experiment
        self.rows = []
        self.images = []
        self._step = 0

    def validate_step(self, step):
        if step < 0 or step > self._step:
            return False
        return True

    def update_metrics(self, client, row):
        """
        Update row in history
        """
        plot_metrics: Dict[str, List[ProtoExperimentPlotMetric]] = {}
        for k, v in row.items():
            if k in plot_metrics:
                plot_metrics[k].append(ProtoExperimentPlotMetric(
                    step=self._step,
                    timestamp=datetime.utcnow().timestamp(),
                    value=float(v),
                ))
            else:
                plot_metrics[k] = [ProtoExperimentPlotMetric(
                    step=self._step,
                    timestamp=datetime.utcnow().timestamp(),
                    value=float(v),
                )]

        self.rows.append(row)
        self._step += 1
        client.experiment_plots_metrics_update(self.experiment.id, plot_metrics)

    def update_images(self, client, row):
        """
        Update images in history
        """
        if not self.experiment.get_plot_volume_id():
            raise ArgumentException("Experiment plot volume id is not set")

        source_path = SAVVIHUB_IMAGES + '/'
        dest_volume_id = self.experiment.get_plot_volume_id()
        dest_snapshot = 'latest'
        dest_path = source_path

        for images in row.values():
            self.images = self.images + images

        responses = volume_file_copy_local_to_remote(
            client,
            source_path=source_path,
            dest_volume_id=dest_volume_id,
            dest_path=dest_path,
            dest_snapshot=dest_snapshot,
            recursive=True
        )

        captions = []
        for images in row.values():
            for image in images:
                captions.append(image.get_caption())
                image.flush()

        plot_files = [ProtoExperimentPlotFile(
            step=None,
            path=response.path,
            caption=caption,
            timestamp=datetime.utcnow().timestamp(),
        ) for response, caption in zip(responses, captions)]
        client.experiment_plots_files_update(self.experiment.id, plot_files, SAVVIHUB_PLOTS_FILETYPE_IMAGE)



import os

from requests_futures.sessions import FuturesSession

from savvihub.api.file_object import UploadableFileObject, DownloadableFileObject
from savvihub.common.utils import wait_all_futures, calculate_crc32c


def default_hooks():
    def fn(resp, **kwargs):
        resp.raise_for_status()
    return {
        'response': fn,
    }


class Downloader:
    @classmethod
    def download(cls, local_path, file, progressable=None):
        dirname = os.path.dirname(local_path)
        basename = os.path.basename(local_path)
        d = DownloadableFileObject(file.download_url.url, dirname, basename)
        d.download(progressable=progressable)

    @classmethod
    def bulk_download(cls, local_base_path, remote_files, progressable=None):
        if len(remote_files) <= 0:
            return
        session = FuturesSession(max_workers=os.environ.get('SAVVIHUB_PARALLEL', 20))
        futures = []
        for remote_file in remote_files:
            if remote_file.path.endswith('/'):
                continue

            d = DownloadableFileObject(remote_file.download_url.url, local_base_path, remote_file.path)
            futures.append(d.download(session, progressable=progressable))

        wait_all_futures(futures)


class Uploader:
    @classmethod
    def get_files_to_upload(cls, local_base_path, hashmap=None):
        local_base_path = local_base_path.rstrip('/')
        results = []
        for root, dirs, files in os.walk(local_base_path):
            for name in files:
                name = os.path.join(os.path.abspath(root), name)
                name = name[len(local_base_path) + 1:] if name.startswith(local_base_path) else name
                if hashmap and hashmap[name] == calculate_crc32c(os.path.join(local_base_path, name)):
                    continue
                results.append(name)
        return results

    @classmethod
    def get_hashmap(cls, local_base_path):
        files = cls.get_files_to_upload(local_base_path)
        hashmap = dict()
        for file in files:
            path = os.path.join(local_base_path, file)
            hashmap[file] = calculate_crc32c(path)
        return hashmap

    @classmethod
    def upload(cls, client, local_path, volume_id, remote_path, progressable=None):
        file_object = client.volume_file_create(volume_id, remote_path, is_dir=False)

        dirname = os.path.dirname(local_path)
        basename = os.path.basename(local_path)
        u = UploadableFileObject(file_object.upload_url.url, dirname, basename)
        u.upload(progressable=progressable)
        resp = client.volume_file_uploaded(volume_id, remote_path)
        return resp

    @classmethod
    def bulk_upload(cls, client, local_base_path, local_file_paths, volume_id, remote_base_path, *, progressable=None):
        # TODO: parallel upload
        if len(local_file_paths) <= 0:
            return

        file_objects = []
        for local_file_path in local_file_paths:
            file_objects.append(client.volume_file_create(
                volume_id,
                os.path.join(remote_base_path, local_file_path),
                is_dir=False,
                # hooks=default_hooks()
            ))

        for i, file_object in enumerate(file_objects):
            u = UploadableFileObject(file_object.upload_url.url, local_base_path, local_file_paths[i])
            u.upload()

        for local_file_path in local_file_paths:
            client.volume_file_uploaded(
                volume_id,
                os.path.join(remote_base_path, local_file_path),
                # hooks=default_hooks(),
            )

        return file_objects



import shortuuid


def get_type_name(obj):
    type_name = obj.__class__.__module__ + "." + obj.__class__.__name__
    if type_name in ["builtins.module", "__builtin__.module"]:
        return obj.__name__
    else:
        return type_name


def generate_uuid():
    generated_uuid = shortuuid.ShortUUID(alphabet=list("0123456789abcdefghijklmnopqrstuvwxyz"))
    return generated_uuid.random(8)



import json
import sys

from openapi_client import ApiException as OpenAPIException
from savvihub.exceptions import SavviHubException


class APIException(SavviHubException):
    error_code = None
    fields = None

    def __init__(self, message, fields=None):
        if self.message is None:
            if message:
                self.message = message

            if fields:
                self.fields = fields
                self.message += ' ('
                for i, field in enumerate(fields):
                    if i > 0:
                        self.message += ', '
                    self.message += field['name']
                    if field['value']:
                        self.message += f': {field["value"]}'
                self.message += ')'

        super().__init__(self.message)

    def __eq__(self, other):
        return self.error_code == other.error_code


class DuplicateAPIException(APIException):
    error_code = 'Duplicate'


class InvalidParametersAPIException(APIException):
    error_code = 'InvalidParameters'


class InvalidTokenAPIException(APIException):
    error_code = 'InvalidToken'
    message = 'Token expired. You should run `sv login` first.'


class NotFoundAPIException(APIException):
    error_code = 'NotFound'


class UnexpectedProblemAPIException(APIException):
    error_code = 'UnexpectedProblem'


class NotADirectoryAPIException(APIException):
    error_code = 'NotADirectory'


class NoSuchFileOrDirectoryAPIException(APIException):
    error_code = 'NoSuchFileOrDirectory'


class K8SUnauthorizedAPIException(APIException):
    error_code = 'K8sUnauthorized'


class K8SPermissionDeniedAPIException(APIException):
    error_code = 'K8sPermissionDenied'


class K8STimeoutAPIException(APIException):
    error_code = 'K8sTimeout'


class ConnectionRefusedAPIException(APIException):
    error_code = 'ConnectionRefused'


class InvalidCACertAPIException(APIException):
    error_code = 'InvalidCACert'


class NetworkTimeoutAPIException(APIException):
    error_code = 'NetworkTimeout'


def inheritors(klass):
    subclasses = []
    work = [klass]
    while work:
        parent = work.pop()
        for child in parent.__subclasses__():
            if child not in subclasses:
                subclasses.append(child)
                work.append(child)
    return subclasses


def convert_to_savvihub_exception(open_api_exception: OpenAPIException) -> APIException:
    try:
        json_resp = json.loads(open_api_exception.body)
        code = json_resp.get('code')
        message = json_resp.get('message')
        fields = json_resp.get('fields')

        for klass in inheritors(APIException):
            if klass.error_code == code:
                return klass(message, fields)

        return UnexpectedProblemAPIException(message, fields)

    except:
        return UnexpectedProblemAPIException(sys.exc_info()[0])



# coding: utf-8

# flake8: noqa
"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from openapi_client.models.access_token_create_api_payload import AccessTokenCreateAPIPayload
from openapi_client.models.account_access_token_create_response import AccountAccessTokenCreateResponse
from openapi_client.models.account_invitation_token_validate_response import AccountInvitationTokenValidateResponse
from openapi_client.models.account_region_list_response import AccountRegionListResponse
from openapi_client.models.account_sign_in_cli_check_response import AccountSignInCliCheckResponse
from openapi_client.models.account_sign_in_cli_token_response import AccountSignInCliTokenResponse
from openapi_client.models.change_password_api_payload import ChangePasswordAPIPayload
from openapi_client.models.change_username_api_payload import ChangeUsernameAPIPayload
from openapi_client.models.cli_experiment_plots_files_update_api_payload import CliExperimentPlotsFilesUpdateAPIPayload
from openapi_client.models.cli_experiment_plots_metrics_update_api_payload import CliExperimentPlotsMetricsUpdateAPIPayload
from openapi_client.models.cluster_all_cluster_list_response import ClusterAllClusterListResponse
from openapi_client.models.cluster_custom_cluster_list_response import ClusterCustomClusterListResponse
from openapi_client.models.cluster_custom_cluster_node_list_response import ClusterCustomClusterNodeListResponse
from openapi_client.models.custom_cluster_add_api_payload import CustomClusterAddAPIPayload
from openapi_client.models.custom_cluster_update_api_payload import CustomClusterUpdateAPIPayload
from openapi_client.models.dataset_update_api_payload import DatasetUpdateAPIPayload
from openapi_client.models.experiment_create_api_payload import ExperimentCreateAPIPayload
from openapi_client.models.experiment_experiment_list_by_project_response import ExperimentExperimentListByProjectResponse
from openapi_client.models.experiment_experiment_list_response import ExperimentExperimentListResponse
from openapi_client.models.experiment_experiment_log_response import ExperimentExperimentLogResponse
from openapi_client.models.experiment_experiment_parameter_list_response import ExperimentExperimentParameterListResponse
from openapi_client.models.experiment_experiment_plots_response import ExperimentExperimentPlotsResponse
from openapi_client.models.experiment_experiment_system_metrics_response import ExperimentExperimentSystemMetricsResponse
from openapi_client.models.experiment_update_api_payload import ExperimentUpdateAPIPayload
from openapi_client.models.external_git_hub_config_response import ExternalGitHubConfigResponse
from openapi_client.models.external_git_hub_repository_list_response import ExternalGitHubRepositoryListResponse
from openapi_client.models.external_google_config_response import ExternalGoogleConfigResponse
from openapi_client.models.external_slack_config_response import ExternalSlackConfigResponse
from openapi_client.models.gs_dataset_create_api_payload import GSDatasetCreateAPIPayload
from openapi_client.models.git_hub_authorize_api_payload import GitHubAuthorizeAPIPayload
from openapi_client.models.github_code_of_conduct import GithubCodeOfConduct
from openapi_client.models.github_license import GithubLicense
from openapi_client.models.github_match import GithubMatch
from openapi_client.models.github_organization import GithubOrganization
from openapi_client.models.github_plan import GithubPlan
from openapi_client.models.github_repository import GithubRepository
from openapi_client.models.github_text_match import GithubTextMatch
from openapi_client.models.github_user import GithubUser
from openapi_client.models.invitation_token_validate_api_payload import InvitationTokenValidateAPIPayload
from openapi_client.models.kernel_kernel_image_list_response import KernelKernelImageListResponse
from openapi_client.models.kernel_kernel_resource_spec_list_response import KernelKernelResourceSpecListResponse
from openapi_client.models.model_access_token import ModelAccessToken
from openapi_client.models.model_access_token_edges import ModelAccessTokenEdges
from openapi_client.models.model_env_var import ModelEnvVar
from openapi_client.models.model_env_vars import ModelEnvVars
from openapi_client.models.model_experiment_metrics_summary import ModelExperimentMetricsSummary
from openapi_client.models.model_file_metadata import ModelFileMetadata
from openapi_client.models.model_kernel_cluster import ModelKernelCluster
from openapi_client.models.model_kernel_cluster_edges import ModelKernelClusterEdges
from openapi_client.models.model_kernel_cluster_node import ModelKernelClusterNode
from openapi_client.models.model_kernel_cluster_node_edges import ModelKernelClusterNodeEdges
from openapi_client.models.model_kernel_cluster_select_policies import ModelKernelClusterSelectPolicies
from openapi_client.models.model_kernel_cluster_select_policy import ModelKernelClusterSelectPolicy
from openapi_client.models.model_kernel_cluster_storage import ModelKernelClusterStorage
from openapi_client.models.model_kernel_cluster_storage_edges import ModelKernelClusterStorageEdges
from openapi_client.models.model_kernel_image import ModelKernelImage
from openapi_client.models.model_kernel_image_edges import ModelKernelImageEdges
from openapi_client.models.model_kernel_image_pull_secret import ModelKernelImagePullSecret
from openapi_client.models.model_kernel_image_pull_secret_edges import ModelKernelImagePullSecretEdges
from openapi_client.models.model_kernel_resource_spec import ModelKernelResourceSpec
from openapi_client.models.model_kernel_resource_spec_field import ModelKernelResourceSpecField
from openapi_client.models.model_notification_config import ModelNotificationConfig
from openapi_client.models.model_notification_config_edges import ModelNotificationConfigEdges
from openapi_client.models.model_page_info_with_count import ModelPageInfoWithCount
from openapi_client.models.model_permission_data import ModelPermissionData
from openapi_client.models.model_permission_detail import ModelPermissionDetail
from openapi_client.models.model_permissions import ModelPermissions
from openapi_client.models.model_project import ModelProject
from openapi_client.models.model_project_edges import ModelProjectEdges
from openapi_client.models.model_service_config import ModelServiceConfig
from openapi_client.models.model_service_config_jupyter_lab import ModelServiceConfigJupyterLab
from openapi_client.models.model_service_config_tensorboard import ModelServiceConfigTensorboard
from openapi_client.models.model_service_endpoint import ModelServiceEndpoint
from openapi_client.models.model_service_port import ModelServicePort
from openapi_client.models.model_service_ports import ModelServicePorts
from openapi_client.models.model_storage import ModelStorage
from openapi_client.models.model_storage_edges import ModelStorageEdges
from openapi_client.models.model_user import ModelUser
from openapi_client.models.model_user_edges import ModelUserEdges
from openapi_client.models.model_user_workspace import ModelUserWorkspace
from openapi_client.models.model_user_workspace_edges import ModelUserWorkspaceEdges
from openapi_client.models.model_volume import ModelVolume
from openapi_client.models.model_volume_edges import ModelVolumeEdges
from openapi_client.models.model_volume_mount import ModelVolumeMount
from openapi_client.models.model_volume_mount_field import ModelVolumeMountField
from openapi_client.models.model_volume_source_dataset import ModelVolumeSourceDataset
from openapi_client.models.model_volume_source_host_path import ModelVolumeSourceHostPath
from openapi_client.models.model_volume_source_nfs import ModelVolumeSourceNFS
from openapi_client.models.model_volume_source_project import ModelVolumeSourceProject
from openapi_client.models.model_volume_source_volume import ModelVolumeSourceVolume
from openapi_client.models.model_workspace import ModelWorkspace
from openapi_client.models.model_workspace_edges import ModelWorkspaceEdges
from openapi_client.models.project_create_api_payload import ProjectCreateAPIPayload
from openapi_client.models.project_git_hub_create_api_payload import ProjectGitHubCreateAPIPayload
from openapi_client.models.project_project_branch_list_response import ProjectProjectBranchListResponse
from openapi_client.models.project_project_commit_list_response import ProjectProjectCommitListResponse
from openapi_client.models.project_update_api_payload import ProjectUpdateAPIPayload
from openapi_client.models.proto_branch import ProtoBranch
from openapi_client.models.proto_commit import ProtoCommit
from openapi_client.models.proto_commit_author import ProtoCommitAuthor
from openapi_client.models.proto_container_system_metrics import ProtoContainerSystemMetrics
from openapi_client.models.proto_current_system_metric import ProtoCurrentSystemMetric
from openapi_client.models.proto_experiment_plot_file import ProtoExperimentPlotFile
from openapi_client.models.proto_experiment_plot_metric import ProtoExperimentPlotMetric
from openapi_client.models.proto_metric import ProtoMetric
from openapi_client.models.proto_node_system_metrics import ProtoNodeSystemMetrics
from openapi_client.models.proto_system_metric import ProtoSystemMetric
from openapi_client.models.proto_volume_mount_request import ProtoVolumeMountRequest
from openapi_client.models.proto_volume_mount_request_source_dataset import ProtoVolumeMountRequestSourceDataset
from openapi_client.models.proto_volume_mount_request_source_host_path import ProtoVolumeMountRequestSourceHostPath
from openapi_client.models.proto_volume_mount_request_source_nfs import ProtoVolumeMountRequestSourceNFS
from openapi_client.models.proto_volume_mount_request_source_project import ProtoVolumeMountRequestSourceProject
from openapi_client.models.proto_volume_mount_request_source_volume import ProtoVolumeMountRequestSourceVolume
from openapi_client.models.proto_volume_mount_requests import ProtoVolumeMountRequests
from openapi_client.models.proto_workload_log import ProtoWorkloadLog
from openapi_client.models.response_dataset_info import ResponseDatasetInfo
from openapi_client.models.response_dataset_info_list import ResponseDatasetInfoList
from openapi_client.models.response_dataset_source import ResponseDatasetSource
from openapi_client.models.response_experiment import ResponseExperiment
from openapi_client.models.response_experiment_history_info import ResponseExperimentHistoryInfo
from openapi_client.models.response_experiment_info import ResponseExperimentInfo
from openapi_client.models.response_experiment_plot_file import ResponseExperimentPlotFile
from openapi_client.models.response_file_metadata import ResponseFileMetadata
from openapi_client.models.response_kernel_cluster import ResponseKernelCluster
from openapi_client.models.response_kernel_cluster_info import ResponseKernelClusterInfo
from openapi_client.models.response_kernel_cluster_node import ResponseKernelClusterNode
from openapi_client.models.response_kernel_cluster_node_info import ResponseKernelClusterNodeInfo
from openapi_client.models.response_kernel_cluster_node_info_experiments_status_summary import ResponseKernelClusterNodeInfoExperimentsStatusSummary
from openapi_client.models.response_kernel_cluster_system_metrics import ResponseKernelClusterSystemMetrics
from openapi_client.models.response_kernel_image import ResponseKernelImage
from openapi_client.models.response_kernel_resource_spec import ResponseKernelResourceSpec
from openapi_client.models.response_my_user import ResponseMyUser
from openapi_client.models.response_project import ResponseProject
from openapi_client.models.response_project_experiment_summary import ResponseProjectExperimentSummary
from openapi_client.models.response_project_info import ResponseProjectInfo
from openapi_client.models.response_project_link import ResponseProjectLink
from openapi_client.models.response_project_list_response import ResponseProjectListResponse
from openapi_client.models.response_region import ResponseRegion
from openapi_client.models.response_service import ResponseService
from openapi_client.models.response_service_info import ResponseServiceInfo
from openapi_client.models.response_service_template import ResponseServiceTemplate
from openapi_client.models.response_service_template_info import ResponseServiceTemplateInfo
from openapi_client.models.response_snapshot import ResponseSnapshot
from openapi_client.models.response_snapshot_info import ResponseSnapshotInfo
from openapi_client.models.response_user import ResponseUser
from openapi_client.models.response_user_with_token_response import ResponseUserWithTokenResponse
from openapi_client.models.response_volume import ResponseVolume
from openapi_client.models.response_volume_mount_info import ResponseVolumeMountInfo
from openapi_client.models.response_volume_mount_infos import ResponseVolumeMountInfos
from openapi_client.models.response_volume_source_dataset import ResponseVolumeSourceDataset
from openapi_client.models.response_volume_source_host_path import ResponseVolumeSourceHostPath
from openapi_client.models.response_volume_source_nfs import ResponseVolumeSourceNFS
from openapi_client.models.response_volume_source_project import ResponseVolumeSourceProject
from openapi_client.models.response_volume_source_volume import ResponseVolumeSourceVolume
from openapi_client.models.response_workspace import ResponseWorkspace
from openapi_client.models.response_workspace_info import ResponseWorkspaceInfo
from openapi_client.models.response_workspace_member import ResponseWorkspaceMember
from openapi_client.models.response_workspace_notification_config import ResponseWorkspaceNotificationConfig
from openapi_client.models.s3_dataset_create_api_payload import S3DatasetCreateAPIPayload
from openapi_client.models.savvi_hub_dataset_create_api_payload import SavviHubDatasetCreateAPIPayload
from openapi_client.models.service_config_update_api_payload import ServiceConfigUpdateAPIPayload
from openapi_client.models.service_create_api_payload import ServiceCreateAPIPayload
from openapi_client.models.service_name_update_api_payload import ServiceNameUpdateAPIPayload
from openapi_client.models.service_resource_spec_update_api_payload import ServiceResourceSpecUpdateAPIPayload
from openapi_client.models.service_service_list_response import ServiceServiceListResponse
from openapi_client.models.service_service_log_response import ServiceServiceLogResponse
from openapi_client.models.service_service_system_metrics_response import ServiceServiceSystemMetricsResponse
from openapi_client.models.service_service_template_list_response import ServiceServiceTemplateListResponse
from openapi_client.models.sign_in_api_payload import SignInAPIPayload
from openapi_client.models.sign_in_cli_confirm_api_payload import SignInCliConfirmAPIPayload
from openapi_client.models.sign_in_git_hub_api_payload import SignInGitHubAPIPayload
from openapi_client.models.sign_in_google_api_payload import SignInGoogleAPIPayload
from openapi_client.models.sign_up_api_payload import SignUpAPIPayload
from openapi_client.models.sign_up_validate_api_payload import SignUpValidateAPIPayload
from openapi_client.models.slack_authorize_api_payload import SlackAuthorizeAPIPayload
from openapi_client.models.snapshot_create_api_payload import SnapshotCreateAPIPayload
from openapi_client.models.snapshot_snapshot_list_response import SnapshotSnapshotListResponse
from openapi_client.models.snapshot_update_api_payload import SnapshotUpdateAPIPayload
from openapi_client.models.storage_file_action_url_info import StorageFileActionURLInfo
from openapi_client.models.verify_email_api_payload import VerifyEmailAPIPayload
from openapi_client.models.volume_file_copy_api_payload import VolumeFileCopyAPIPayload
from openapi_client.models.volume_file_create_api_payload import VolumeFileCreateAPIPayload
from openapi_client.models.volume_volume_file_copy_response import VolumeVolumeFileCopyResponse
from openapi_client.models.volume_volume_file_delete_response import VolumeVolumeFileDeleteResponse
from openapi_client.models.volume_volume_file_list_response import VolumeVolumeFileListResponse
from openapi_client.models.workspace_create_api_payload import WorkspaceCreateAPIPayload
from openapi_client.models.workspace_member_add_api_payload import WorkspaceMemberAddAPIPayload
from openapi_client.models.workspace_notification_config_email_add_api_payload import WorkspaceNotificationConfigEmailAddAPIPayload
from openapi_client.models.workspace_update_api_payload import WorkspaceUpdateAPIPayload
from openapi_client.models.workspace_workspace_list_response import WorkspaceWorkspaceListResponse
from openapi_client.models.workspace_workspace_member_list_response import WorkspaceWorkspaceMemberListResponse



class SavviHubException(BaseException):
    message = None
    exit_code = 1

    def __init__(self, message=None, exit_code=1):
        self.exit_code = exit_code
        if message:
            self.message = message


class PathException(SavviHubException):
    pass


class ArgumentException(SavviHubException):
    pass



import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Context
from savvihub.cli.utils import get_default_workspace


def workspace_name_callback(ctx: Context, workspace_name: str) -> str:
    if workspace_name:
        try:
            ctx.authenticated_client.workspace_read(workspace_name)
        except NotFoundAPIException:
            raise ExitException('Workspace not found.')
        return workspace_name

    return (ctx.workspace.name if ctx.workspace
            else get_default_workspace(ctx.authenticated_client).name)


workspace_name_option = typer.Option(None, '--workspace', callback=workspace_name_callback,
                                     help='Override the default workspace.')



import inquirer
import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.exceptions import ExitException, InvalidGitRepository
from savvihub.cli.git import GitRepository
from savvihub.cli.typer import Context


def project_name_callback(ctx: Context, project_name: str) -> str:
    workspace_name = ctx.params['workspace_name']
    try:
        git_repo = GitRepository()
        if not project_name:
            project_name = git_repo._get_github_repo()[1]
            ctx.git_repo = git_repo
        elif project_name == git_repo._get_github_repo()[1]:
            ctx.git_repo = git_repo
    except InvalidGitRepository:
        if not project_name:
            projects = ctx.authenticated_client.project_list(workspace_name).results
            if len(projects) == 1:
                ctx.project = projects[0]
            else:
                ctx.project = inquirer.prompt([inquirer.List(
                    'project',
                    message='Select project',
                    choices=[(p.name, p) for p in projects],
                )]).get('project')

            return ctx.project.name

    try:
        ctx.project = ctx.authenticated_client.project_read(workspace_name, project_name)
    except NotFoundAPIException:
        raise ExitException(f'Project `{project_name}` does not exist in the workspace `{workspace_name}`.')

    return project_name


project_name_option = typer.Option(None, '--project', callback=project_name_callback,
                                   help='If not present, uses git repository name of the current directory.')



import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.commands.volume import volume_file_list, volume_file_remove, volume_file_copy
from savvihub.cli.exceptions import ExitException
from savvihub.cli.inputs.workspace import workspace_name_option
from savvihub.cli.typer import Typer, Context
from savvihub.cli.utils import parse_dataset

dataset_files_app = Typer()


@dataset_files_app.callback()
def main():
    """
    Manage files in the dataset
    """
    return


@dataset_files_app.command(user_required=True)
def ls(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET',
                                            help='{workspace}/{dataset}@{snapshot} format '
                                                 'where workspace and snapshot are optional.'),
    path: str = typer.Argument('/'),
    workspace_name: str = workspace_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive', help='recursive flag'),
    directory: bool = typer.Option(False, '-d', '--directory', help='list the directory itself, not its contents')
):
    """
    List files in the dataset with prefix
    """
    workspace_name_override, dataset_name, snapshot_name = parse_dataset(dataset_full_name)
    if workspace_name_override:
        workspace_name = workspace_name_override

    dataset = ctx.authenticated_client.dataset_read(workspace_name, dataset_name)
    volume_file_list(ctx, dataset.volume_id, snapshot_name, path, recursive, directory)


@dataset_files_app.command(user_required=True)
def rm(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET_NAME'),
    path: str = typer.Argument(...),
    workspace_name: str = workspace_name_option,
    recursive: bool = typer.Option(False, '-r', '-R', '--recursive',
                                   help='Remove directories and their contents recursively'),
):
    """
    Remove files in a dataset (SavviHub dataset files only)
    """
    workspace_name_override, dataset_name, snapshot_name = parse_dataset(dataset_full_name)
    if workspace_name_override:
        workspace_name = workspace_name_override

    if snapshot_name != 'latest':
        raise ExitException('Files in a snapshot cannot be removed.')

    dataset = ctx.authenticated_client.dataset_read(workspace_name, dataset_name)
    volume_file_remove(ctx, dataset.volume_id, path, recursive)


@dataset_files_app.command(user_required=True)
def upload(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET_NAME'),
    source_path: str = typer.Argument(...),
    dest_path: str = typer.Argument(...),
    workspace_name: str = workspace_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Upload files to a dataset (SavviHub dataset only)
    """
    workspace_name_override, dataset_name, snapshot_name = parse_dataset(dataset_full_name)
    if workspace_name_override:
        workspace_name = workspace_name_override

    if snapshot_name != 'latest':
        raise ExitException('Files in a snapshot cannot be removed.')

    dataset = ctx.authenticated_client.dataset_read(workspace_name, dataset_name)
    volume_file_copy(
        ctx,
        source_volume_id=None,
        source_snapshot=None,
        source_path=source_path,
        dest_volume_id=dataset.volume_id,
        dest_snapshot='latest',
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )


@dataset_files_app.command(user_required=True)
def download(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET',
                                            help='{workspace}/{dataset}@{snapshot} format '
                                                 'where workspace and snapshot are optional.'),
    source_path: str = typer.Argument(...),
    dest_path: str = typer.Argument('.'),
    workspace_name: str = workspace_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Download files from a dataset
    """
    workspace_name_override, dataset_name, snapshot_name = parse_dataset(dataset_full_name)
    if workspace_name_override:
        workspace_name = workspace_name_override

    dataset = ctx.authenticated_client.dataset_read(workspace_name, dataset_name)
    if snapshot_name != 'latest':
        try:
            ctx.authenticated_client.snapshot_read(dataset.volume_id, snapshot_name)
        except NotFoundAPIException:
            raise ExitException(f'Invalid dataset snapshot: {dataset_full_name}\n'
                                f'Please check your dataset and snapshot exist in workspace `{workspace_name}`.')
    volume_file_copy(
        ctx,
        source_volume_id=dataset.volume_id,
        source_snapshot=snapshot_name,
        source_path=source_path,
        dest_volume_id=None,
        dest_snapshot=None,
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )


@dataset_files_app.command(user_required=True)
def cp(
    ctx: Context,
    dataset_full_name: str = typer.Argument(..., metavar='DATASET',
                                            help='{workspace}/{dataset}@{snapshot} format '
                                                 'where workspace and snapshot are optional. '
                                                 'If a snapshot is specified, '
                                                 'copies the files from the snapshot to the current dataset.'),
    source_path: str = typer.Argument(...),
    dest_path: str = typer.Argument(...),
    workspace_name: str = workspace_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Copy files within a dataset (SavviHub dataset files only)
    """
    workspace_name_override, dataset_name, src_snapshot_name = parse_dataset(dataset_full_name)
    if workspace_name_override:
        workspace_name = workspace_name_override

    dataset = ctx.authenticated_client.dataset_read(workspace_name, dataset_name)
    if src_snapshot_name != 'latest':
        try:
            ctx.authenticated_client.snapshot_read(dataset.volume_id, src_snapshot_name)
        except NotFoundAPIException:
            raise ExitException(f'Invalid dataset snapshot: {dataset_full_name}\n'
                                f'Please check your dataset and snapshot exist in workspace `{workspace_name}`.')
    volume_file_copy(
        ctx,
        source_volume_id=dataset.volume_id,
        source_snapshot=src_snapshot_name,
        source_path=source_path,
        dest_volume_id=dataset.volume_id,
        dest_snapshot='latest',
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )
